//Phong model fragment shader
#version 330 core

in vec3 worldNormal;
in vec3 worldPos;
in vec2 fragmentUV;
in vec4 viewSpace;
in vec4 projCoords;
in vec4 fragPosLightSpace;


layout (location = 0) out vec4 fragColor;


uniform sampler2D color_texture;

uniform bool project;
uniform sampler2D projected_texture;

uniform sampler2D shadowMap;

uniform vec3 lightPos;
uniform vec3 cameraPos;

uniform int material_shininess;
uniform float material_kd;
uniform float material_ks;


uniform bool singleColor;

uniform bool transparent;

uniform bool fog;
uniform int maxFogDist;

uniform bool absoluteLighting;


//const float ambient = 0.08;
float ambient = 0.2;
const vec4 diffuseColor = vec4(0.15, 0.05, 0.5, 1.0);
const vec4 specularColor = vec4(1.0, 1.0, 1.0, 1.0);

const vec4 fogColor = vec4(0.25, 0.25, 0.5, 1.0);
const float FogDensity = 0.1;

//turn off/on the spotlight
uniform bool spotlight;
uniform float cones;
uniform float outerCones;
uniform vec3 cameraFront;

uniform bool shadows;


float calcShadows(vec4 fragPosLightSpace)
{
  
   // transform coords
    vec3 mapCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    mapCoords = mapCoords * 0.5 + 0.5;

    float closestDepth = texture(shadowMap, mapCoords.xy).r; 
    float currentDepth = mapCoords.z;

    // bias
    vec3 normal = normalize(worldNormal);
    vec3 lightDir = normalize(lightPos - worldPos);
   float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
   //float bias = 0.0f;

     //float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;
	 //float shadow = currentDepth > closestDepth  ? 1.0 : 0.0;
	
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, mapCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
         }    
     }
    shadow /= 9.0;
    

    //  0.0 when outside the far_plane of the light's frustum.
    //if(mapCoords.z > 1.0)
    //    shadow = 0.0;
        
    return shadow;
	//return currentDepth;
	//return closestDepth;
}



void main()
{
	vec4 textureColor = texture(color_texture, fragmentUV);

	if (singleColor) {
		textureColor = vec4(1.0, 1.0, 1.0, 0.0);
	}

	//fragColor = vec4((worldNormal + 1)/2, 1.0); // to represent normals as colors
	//fragColor = vec4(textureColor, 1.0); // texture only

	vec3 L = normalize(lightPos - worldPos); // light direction
	vec3 V = normalize(cameraPos - worldPos); // viewing direction
 
	float LdotN = max(0, dot(L,normalize(worldNormal)));
 
	float diffuse = 0.0f;
	diffuse = material_kd * LdotN; // diffuse light intensity
	
	//vec4 diffuseLight = diffuseColor * diffuse;
 
	float specular = 0.0f;

	if(LdotN > 0.0)
	{
		//vec3 R = -normalize(reflect(L, worldNormal));//Reflection
		//specular = material_ks * pow(max(0.0, dot(R, V)), material_shininess);
 
		// Blinn-Phong Lighting Model
		vec3 H = normalize(L + V ); // Halfway Vector
		specular = material_ks * pow(max(0, dot(H, normalize(worldNormal))), material_shininess); // specular light
	}

	//vec3 specularLight = specularColor * specular;
	//vec3 ambientLight = vec3(ambient, ambient, ambient);
 
	//float light = ambient + diffuse + specular;
	//float light = max(ambient, diffuse + specular);


	if(spotlight){
		float theta = dot(V, normalize(-cameraFront));
		float attenuation = cones - outerCones;
		float attenuationValue = (theta - outerCones)/attenuation;
		if(attenuationValue < 0.3){
			attenuationValue = 0.3;
		}else if(attenuationValue > 1.0){
			attenuationValue = 1.0;
		}
		if(theta > outerCones){
			diffuse = diffuse * attenuationValue;
			specular = specular * attenuationValue;
			diffuse = diffuse * 4;
			specular = specular * 4;
			//ambient = ambient * 2;
			//textureLightColor = vec4(textureLightColor.xyz * intensity,fragColor.w);
			//textureLightColor = vec4((textureLightColor.xyz)*1.8,fragColor.w);
			//textureLightColor = vec4(1.0, 0.0, 0.0,fragColor.w);
			//maxFogDist_ = maxFogDist * 2;
		}
		
	}

	float light = max(ambient, ambient + (diffuse + specular));

	if (!((ambient + diffuse + specular) < 0.5) && !((ambient + diffuse + specular) > 0.5)) {
	light = ambient;}

	if (absoluteLighting) {
		light = 1.0;
	}

	//vec3 lightColor = ambientLight + diffuseLight + specularLight + textureColor;

	vec4 textureLightColor = light * textureColor;


 
	if (project) { // add projected texture, if projection mode is turned on
		vec2 finalCoords    = projCoords.st / projCoords.q;
		vec4 vProjTexColor  = texture(projected_texture, finalCoords);

		if (projCoords.q > 0.0 && worldNormal.y > 0.0) { // only show projection on the upper side
			textureLightColor = worldNormal.y*0.1*vProjTexColor + textureLightColor;
		} 
   	}


	// Calculate shadow
    float shadow = shadows ? calcShadows(fragPosLightSpace) : 0.0;                      
    shadow = min(shadow, 0.75); // reduce shadow strength a little: allow some diffuse/specular light in shadowed regions
    textureLightColor = (1.0 - shadow) * textureLightColor;
	

	float maxFogDist_ = maxFogDist;

	if (fog) {
		// add fog:
		float fogFactor = 0;

		float dist = 0; // distance from the viewer
		dist = length(viewSpace);

 		// calculate linear fog
		//fogFactor = (20 - dist)/(20 - 10);
		fogFactor = (maxFogDist_ - dist)/(maxFogDist_ - 10);
   		fogFactor = clamp(fogFactor, 0.0, 1.0);

		textureLightColor = mix(fogColor, textureLightColor, fogFactor);
	}


	if (transparent) {
		//textureLightColor.a = 0.3;
		textureLightColor.a = min(textureLightColor.b, 0.3);
	}


	fragColor = textureLightColor;
	//fragColor = vec4(vec3(shadow), 1.0);
	fragColor = fragColor * 1.3f;
}
