//Phong model fragment shader
#version 330 core

in vec3 worldNormal;
in vec3 worldPos;
in vec2 fragmentUV;
in vec4 viewSpace;

layout (location = 0) out vec4 fragColor;

uniform sampler2D color_texture;

uniform vec3 lightPos;
uniform vec3 cameraPos;

uniform int material_shininess;
uniform float material_kd;
uniform float material_ks;

const float ambient = 0.08;
const vec3 diffuseColor = vec3(0.15, 0.05, 0.5);
const vec3 specularColor = vec3(1.0, 1.0, 1.0);

const vec3 fogColor = vec3(0.25, 0.25, 0.5);
const float FogDensity = 0.1;

void main()
{
	vec3 textureColor = texture(color_texture, fragmentUV).rgb;
	////fragColor = vec4((worldNormal + 1)/2, 1.0); // to represent normals as colors
	//fragColor = vec4(textureColor, 1.0);


	vec3 L = normalize(lightPos - worldPos); // light direction
	vec3 V = normalize(cameraPos - worldPos); // viewing direction
 
	float LdotN = max(0, dot(L,normalize(worldNormal)));
 
	float diffuse = material_kd * LdotN; // diffuse light intensity
	
	vec3 diffuseLight = diffuseColor * diffuse;
 
	float specular = 0.0;

	if(LdotN > 0.0)
	{
 
		//vec3 R = -normalize(reflect(L, worldNormal));//Reflection
		//specular = material_ks * pow(max(0.0, dot(R, V)), material_shininess);
 
		// Blinn-Phong Lighting Model
		vec3 H = normalize(L + V ); // Halfway Vector
		specular = material_ks * pow(max(0, dot(H, normalize(worldNormal))), material_shininess); // specular light
	}

	//vec3 specularLight = specularColor * specular;
	
	//vec3 ambientLight = vec3(ambient, ambient, ambient);
 
	//float light = ambient + diffuse + specular;

	float light = max(ambient, diffuse + specular);

	//vec3 lightColor = ambientLight + diffuseLight + specularLight + textureColor;


	//fragColor = vec4(light,light, light,1);

	//fragColor = vec4(textureColor.x*light, textureColor.y*light, textureColor.z*light,1);
	//vec3 textureLightColor = vec3(textureColor.x*light, textureColor.y*light, textureColor.z*light);
	vec3 textureLightColor = light * textureColor;

	//vec3 textureLightColor = vec3(specular);	

	//fragColor = vec4(light, light, light, light);
	//fragColor = vec4(lightColor, 1);

	//distance
	float dist = 0;
	float fogFactor = 0;

	dist = length(viewSpace);

 	// calculate linear fog
	fogFactor = (20 - dist)/(20 - 10);
   	fogFactor = clamp(fogFactor, 0.0, 1.0);
 
   	fragColor = vec4(mix(fogColor, textureLightColor, fogFactor),1);

}
